# frozen_string_literal: true

module SafetyNetAttestation
  GEM_ROOT = File.expand_path(__dir__)
end

require_relative "safety_net_attestation/statement"
require_relative "safety_net_attestation/version"
